==> autogrid-1.0.0-alpha1.tar.gz: autogrid/src/Controller/GridController.php <==
<?php

namespace Drupal\autogrid\Controller;

/**
 * @file
 * Contains \Drupal\autogrid\Controller\GridController.
 */

use Drupal\Core\Cache\Cache;
use Drupal\Core\Config\Entity\ConfigEntityType;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Pager\PagerManagerInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Contains the primary entity relationship diagram for this module.
 */
class GridController extends ControllerBase {

  /**
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected $entityTypeBundleInfo;

  /**
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The pager manager.
   *
   * @var \Drupal\Core\Pager\PagerManagerInterface
   */
  protected $pagerManager;

  /**
   * The set of fields to be output.
   *
   * @var array
   */
  protected $fields;

  /**
   * An EntityListBuilder to create operation links.
   *
   * @var \Drupal\Core\Entity\EntityListBuilder
   */
  protected $entityListBuilder;

  /**
   * Constructs a new GridController.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The EntityTypeManager utility class.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The EntityFieldManager utility class.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_type_bundle_info
   *   The EntityTypeBundleInfo utility class.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The EntityTypeManager utility class.
   * @param \Drupal\Core\Pager\PagerManagerInterface $pager_manager
   *   The EntityTypeManager utility class.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, EntityFieldManagerInterface $entity_field_manager, EntityTypeBundleInfoInterface $entity_type_bundle_info, ModuleHandlerInterface $module_handler, PagerManagerInterface $pager_manager) {
    $this->entityTypeManager = $entity_type_manager;
    $this->entityFieldManager = $entity_field_manager;
    $this->entityTypeBundleInfo = $entity_type_bundle_info;
    $this->moduleHandler = $module_handler;
    $this->pagerManager = $pager_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('entity_field.manager'),
      $container->get('entity_type.bundle.info'),
      $container->get('module_handler'),
      $container->get('pager.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header = [];
    foreach ($this->fields as $field_name => $field) {
      $header[$field_name] = [
        'data' => $field->getLabel(),
        'field' => $field_name,
        'specifier' => $field_name,
      ];
    }
    $header['ops'] = $this->t('Operations');
    return $header;
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    foreach ($this->fields as $field_name => $field) {
      $row[$field_name]['data'] = $entity->get($field_name)->view();
      $row[$field_name]['data']['#label_display'] = 'hidden';
    }
    $row['ops']['data'] = $this->entityListBuilder->buildOperations($entity);
    return $row;
  }

  /**
   * Generate the markup to display for the provided entity type and bundle.
   *
   * @param string $entity_type_id
   *   Machine name of the entity type.
   * @param string $bundle
   *   Name of the bundle whose content will be displayed.
   *
   * @return array
   *   Render array to be displayed.
   */
  public function getTable($entity_type_id, $bundle) {
    // @todo make the pager length configurable.
    $pager = 10;

    // Build the header first so we can sort on columns.
    $fields = $this->entityFieldManager->getFieldDefinitions($entity_type_id, $bundle);
    // Include the id, which can have different names.
    $include[] = array_key_first($fields);
    foreach ($fields as $field_name => $field_definition) {
      if (strpos($field_name, 'field_') !== 0 && !in_array($field_name, $include)) {
        unset($fields[$field_name]);
      }
    }
    $this->fields = $fields;

    $header = $this->buildHeader();

    $query = \Drupal::entityQuery($entity_type_id)
      ->tableSort($header)
      ->condition('type', $bundle)
      ->pager($pager);
    $results = $query->execute();
    if (!$results) {
      return [
        '#type' => 'markup',
        '#markup' => '<p>' . $this->t('No matching results found.') . '</p>',
      ];
    }
    $storage = $this->entityTypeManager->getStorage($entity_type_id);
    $entities = $storage->loadMultiple($results);
    $this->entityListBuilder = $this->entityTypeManager->getListBuilder($entity_type_id);

    // @todo make display of base fields configurable.
    $include = ['body', 'title', 'name'];
    $rows = [];
    foreach ($entities as $entity) {
      $rows[] = $this->buildRow($entity);
    }

    // Invalidate cache if content in this bundle or the bundle config changes.
    $tags = [
      $entity_type_id . '_list:' . $bundle,
      'config:' . $entity_type_id . '_type:' . $bundle,
    ];

    return [
      'results' => [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $rows,
      ],
      'pager' => [
        '#type' => 'pager',
      ],
      '#cache' => [
        'contexts' => ['languages', 'url.path', 'user'],
        'tags' => $tags,
        'max-age' => Cache::PERMANENT,
      ],
    ];
  }

  /**
   * No parameters provided, so provide a list of content entity types.
   *
   * @return array
   *   The list of links to choose from.
   */
  public function getEntityTypeLinks() {
    $entity_definitions = $this->entityTypeManager->getDefinitions();
    $entities = [];
    foreach ($entity_definitions as $definition_id => $definition) {
      if ($definition instanceof ConfigEntityType) {
        continue;
      }
      $url = Url::fromRoute('autogrid.choose_bundle', ['entity_type_id' => $definition_id]);
      // @todo add access check?
      $entities[] = [
        '#type' => 'link',
        '#url' => $url,
        '#title' => $definition->getLabel(),
      ];
    }
    return [
      '#theme' => 'item_list',
      '#items' => $entities,
    ];
  }

  /**
   * No bundle provided, so provide a list of bundles for the entity type.
   *
   * @param string $entity_type_id
   *   The entity whose bundles will be listed.
   *
   * @return array
   *   The list of links to choose from.
   */
  public function getBundleLinks($entity_type_id) {
    $bundles = \Drupal::service('entity_type.bundle.info')->getBundleInfo($entity_type_id);
    $links = [];
    // @todo if one one bundle, redirect automatically.
    foreach ($bundles as $bundle_name => $bundle_info) {
      $url = Url::fromRoute('autogrid.display', ['entity_type_id' => $entity_type_id, 'bundle' => $bundle_name]);
      // @todo add access check?
      $links[] = [
        '#type' => 'link',
        '#url' => $url,
        '#title' => $bundle_info['label'],
      ];
    }
    return [
      '#theme' => 'item_list',
      '#items' => $links,
    ];
  }

  /**
   * Title callback to generate a title based on the parameters.
   *
   * @param string $entity_type_id
   *   Machine name of the entity type.
   * @param string $bundle
   *   Name of the bundle whose content will be displayed.
   *
   * @return string
   *   The title to be used.
   */
  public function getTitle($entity_type_id, $bundle) {
    if (!$entity_type_id || !$bundle) {
      return $this->t('Content Grid');
    }
    $entity_definition = $this->entityTypeManager->getDefinition($entity_type_id);
    $bundles = $this->entityTypeBundleInfo->getBundleInfo($entity_type_id);
    // @todo more conditional logic to allow for missing/invalid parameters.
    return $this->t('All @type @content', ['@type' => $bundles[$bundle]['label'], '@content' => $entity_definition->getPluralLabel()]);
  }

  /**
   * Returns pager array.
   */
  public function pagerArray($items, $itemsPerPage) {
    // Get total items count.
    $total = count($items);
    // Get the number of the current page.
    $currentPage = $this->pagerManager->createPager($total, $itemsPerPage)->getCurrentPage();
    // Split an array into chunks.
    $chunks = array_chunk($items, $itemsPerPage);
    // Return current group item.
    $currentPageItems = $chunks[$currentPage];
    return $currentPageItems;
  }

}

==> autogrid-1.1.0-alpha1.tar.gz: autogrid/src/Controller/GridController.php <==
<?php

namespace Drupal\autogrid\Controller;

/**
 * @file
 * Contains \Drupal\autogrid\Controller\GridController.
 */

use Drupal\Core\Cache\Cache;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\Entity\ConfigEntityType;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Pager\PagerManagerInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Contains the primary entity relationship diagram for this module.
 */
class GridController extends ControllerBase {

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Provides information about bundle configuration.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected $entityTypeBundleInfo;

  /**
   * Provides information about the installed modules.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The entity display repository.
   *
   * @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface
   */
  protected $entityDisplayRepository;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The pager manager.
   *
   * @var \Drupal\Core\Pager\PagerManagerInterface
   */
  protected $pagerManager;

  /**
   * The set of fields to be output.
   *
   * @var array
   */
  protected $fields;

  /**
   * The view mode to be used for rendering content.
   *
   * @var string
   */
  protected $viewMode = 'default';

  /**
   * An EntityListBuilder to create operation links.
   *
   * @var \Drupal\Core\Entity\EntityListBuilder
   */
  protected $entityListBuilder;

  /**
   * Constructs a new GridController.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The EntityTypeManager utility class.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The EntityFieldManager utility class.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_type_bundle_info
   *   The EntityTypeBundleInfo utility class.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The EntityTypeManager utility class.
   * @param \Drupal\Core\Entity\EntityDisplayRepositoryInterface $entity_display_repository
   *   The EntityDisplayRepository utility class.
   * @param \Drupal\Core\Pager\PagerManagerInterface $pager_manager
   *   The EntityTypeManager utility class.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, EntityFieldManagerInterface $entity_field_manager, EntityTypeBundleInfoInterface $entity_type_bundle_info, ModuleHandlerInterface $module_handler, EntityDisplayRepositoryInterface$entity_display_repository, ConfigFactoryInterface $configFactory, PagerManagerInterface $pager_manager) {
    $this->entityTypeManager = $entity_type_manager;
    $this->entityFieldManager = $entity_field_manager;
    $this->entityTypeBundleInfo = $entity_type_bundle_info;
    $this->moduleHandler = $module_handler;
    $this->entityDisplayRepository = $entity_display_repository;
    $this->configFactory = $configFactory;
    $this->pagerManager = $pager_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('entity_field.manager'),
      $container->get('entity_type.bundle.info'),
      $container->get('module_handler'),
      $container->get('entity_display.repository'),
      $container->get('config.factory'),
      $container->get('pager.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader($id_key) {
    $header = [];
    $header['id'] = [
      'data' => $this->t('ID'),
      'field' => $id_key,
      'specifier' => $id_key,
    ];
    foreach ($this->fields as $field_name => $field) {
      $header[$field_name] = [
        'data' => $field->getLabel(),
        'field' => $field_name,
        'specifier' => $field_name,
      ];
    }
    $header['ops'] = $this->t('Operations');
    return $header;
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    $row['id']['data'] = $entity->id();
    foreach ($this->fields as $field_name => $field) {
      if ($entity->get($field_name)->isEmpty()) {
        $row[$field_name]['data'] = '';
      }
      else {
        $row[$field_name]['data'] = $entity->get($field_name)->view($this->viewMode);
        $row[$field_name]['data']['#label_display'] = 'hidden';
      }
    }
    $row['ops']['data'] = $this->entityListBuilder->buildOperations($entity);
    return $row;
  }

  /**
   * Generate the markup to display for the provided entity type and bundle.
   *
   * @param string $entity_type_id
   *   Machine name of the entity type.
   *
   * @return array
   *   Render array to be displayed.
   */
  public function getTable($entity_type_id) {
    // @todo Provide more logic specific for this use case, and move some of the
    // logic in getBundleTabe() into a helper method, instead of calling it.
    return $this->getBundleTable($entity_type_id, $entity_type_id);
  }

  /**
   * Generate the markup to display for the provided entity type and bundle.
   *
   * @param string $entity_type_id
   *   Machine name of the entity type.
   * @param string $bundle
   *   Machine name of the bundle whose content will be displayed.
   *
   * @return array
   *   Render array to be displayed.
   */
  public function getBundleTable($entity_type_id, $bundle) {
    $parent_def = $this->entityTypeManager->getDefinition($entity_type_id);

    $bundle_key = $parent_def->getKey('bundle');
    if (empty($bundle_key) || !$this->entityTypeManager->hasHandler($entity_type_id, 'list_builder')) {
      $this->messenger->addWarning('Unsupported Entity Type.');
      return $this->redirect('autogrid.entities_form');
    }

    // Validate provided bundle key.
    $bundles = $this->entityTypeBundleInfo->getBundleInfo($entity_type_id);
    if (!isset($bundles[$bundle])) {
      $this->messenger->addWarning('Invalid bundle specified.');
      return [];
    }

    // @todo make the pager length configurable.
    $pager = 10;

    $view_mode = $this->configFactory->get('autogrid.entities.' . $entity_type_id . '.' . $bundle)?->get('view_mode') ?? 'default';
    $this->viewMode = $view_mode;

    // Build the header first so we can sort on columns.
    $display = $this->entityDisplayRepository->getViewDisplay($entity_type_id, $bundle, $view_mode);
    $fields = $display->get('fieldDefinitions');
    $display_def = $display->getComponents();
    uasort($display_def, ['Drupal\Component\Utility\SortArray', 'sortByWeightElement']);

    // @todo make display of base fields configurable.
    $exclude = array_merge(['uid', 'created'], array_keys($display->get('hidden')));
    $fields_sorted = [];
    foreach ($display_def as $field_name => $properties) {
      if (in_array($field_name, $exclude) || $display_def[$field_name]['region'] == 'hidden' || !isset($fields[$field_name])) {
        unset($fields[$field_name]);
        continue;
      }
      $fields_sorted[$field_name] = $fields[$field_name];
    }
    $this->fields = $fields_sorted;

    $header = $this->buildHeader($parent_def->getKey('id'));

    $storage = $this->entityTypeManager->getStorage($entity_type_id);
    $query = $storage->getQuery()
      ->tableSort($header)
      ->condition($bundle_key, $bundle)
      ->pager($pager);
    $results = $query->execute();
    if (!$results) {
      return [
        '#type' => 'markup',
        '#markup' => '<p>' . $this->t('No matching results found.') . '</p>',
      ];
    }
    $storage = $this->entityTypeManager->getStorage($entity_type_id);
    $entities = $storage->loadMultiple($results);
    $this->entityListBuilder = $this->entityTypeManager->getListBuilder($entity_type_id);
    $rows = [];
    foreach ($entities as $entity) {
      $rows[] = $this->buildRow($entity);
    }

    // Invalidate cache if content in this bundle or the bundle config changes.
    $tags = [
      $entity_type_id . '_list:' . $bundle,
      'config:' . $entity_type_id . '_type:' . $bundle,
    ];

    return [
      'results' => [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $rows,
      ],
      'pager' => [
        '#type' => 'pager',
      ],
      '#cache' => [
        'contexts' => ['languages', 'url.path', 'user'],
        'tags' => $tags,
        'max-age' => Cache::PERMANENT,
      ],
    ];
  }

  /**
   * No parameters provided, so provide a list of content entity types.
   *
   * @return array
   *   The list of links to choose from.
   */
  public function getEntityTypeLinks() {
    $entity_definitions = $this->entityTypeManager->getDefinitions();
    $entities = [];
    foreach ($entity_definitions as $definition_id => $definition) {
      if ($definition instanceof ConfigEntityType || !$this->entityTypeManager->getDefinition($definition_id)->getKey('bundle') || !$this->entityTypeManager->hasHandler($definition_id, 'list_builder')) {
        continue;
      }
      $url = Url::fromRoute('autogrid.choose_bundle', ['entity_type_id' => $definition_id]);
      // @todo add access check?
      $entities[] = [
        '#type' => 'link',
        '#url' => $url,
        '#title' => $definition->getLabel(),
      ];
    }
    return [
      '#theme' => 'item_list',
      '#items' => $entities,
    ];
  }

  /**
   * No bundle provided, so provide a list of bundles for the entity type.
   *
   * @param string $entity_type_id
   *   The entity whose bundles will be listed.
   *
   * @return array
   *   The list of links to choose from.
   */
  public function getBundleLinks($entity_type_id) {
    $bundle_key = $this->entityTypeManager->getDefinition($entity_type_id)->getKey('bundle');
    if (empty($bundle_key) || !$this->entityTypeManager->hasHandler($entity_type_id, 'list_builder')) {
      $this->messenger->addWarning('Unsupported Entity Type.');
      return $this->redirect('autogrid.choose_entity_type');
    }
    $bundles = $this->entityTypeBundleInfo->getBundleInfo($entity_type_id);
    $links = [];
    foreach ($bundles as $bundle_name => $bundle_info) {
      $url = Url::fromRoute('autogrid.display', ['entity_type_id' => $entity_type_id, 'bundle' => $bundle_name]);
      // @todo add access check?
      $links[] = [
        '#type' => 'link',
        '#url' => $url,
        '#title' => $bundle_info['label'],
      ];
    }
    return [
      '#theme' => 'item_list',
      '#items' => $links,
    ];
  }

  /**
   * Title callback to generate a title based on the parameters.
   *
   * @param string $entity_type_id
   *   Machine name of the entity type.
   *
   * @return string
   *   The title to be used.
   */
  public function getTitle($entity_type_id) {
    if (!$entity_type_id) {
      return $this->t('Content Grid');
    }
    $entity_definition = $this->entityTypeManager->getDefinition($entity_type_id);
    // @todo more conditional logic to allow for missing/invalid parameters.
    return $this->t('All @content', ['@content' => $entity_definition->getPluralLabel()]);
  }

  /**
   * Title callback to generate a title based on the parameters.
   *
   * @param string $entity_type_id
   *   Machine name of the entity type.
   * @param string $bundle
   *   Machine name of the bundle whose content will be displayed.
   *
   * @return string
   *   The title to be used.
   */
  public function getBundleTitle($entity_type_id, $bundle) {
    if (!$entity_type_id || !$bundle) {
      return $this->t('Content Grid');
    }
    $entity_definition = $this->entityTypeManager->getDefinition($entity_type_id);
    $bundles = $this->entityTypeBundleInfo->getBundleInfo($entity_type_id);
    // @todo more conditional logic to allow for missing/invalid parameters.
    return $this->t('All @type @content', ['@type' => $bundles[$bundle]['label'], '@content' => $entity_definition->getPluralLabel()]);
  }

  /**
   * Returns pager array.
   */
  public function pagerArray($items, $itemsPerPage) {
    // Get total items count.
    $total = count($items);
    // Get the number of the current page.
    $currentPage = $this->pagerManager->createPager($total, $itemsPerPage)->getCurrentPage();
    // Split an array into chunks.
    $chunks = array_chunk($items, $itemsPerPage);
    // Return current group item.
    $currentPageItems = $chunks[$currentPage];
    return $currentPageItems;
  }

}
